% Implements transfer function feedback controller, using Jeff Butterworths
% H-infity controller, which was designed for this specific stage some
% years ago. 

clc
clear all
addpath('functions')

% Options
figbase  = 50;

controlParamName = 'exp01Controls.csv'; 
refTrajName      = 'ref_traj_track.csv';

% -------------------------------------------------------------------------
% Load Models

matpath       = getMatPath();
dataroot      = fullfile(matpath, 'AFM_SS', 'System_Identification', 'data', 'data_xaxis'); 
expDataRoot     = fullfile(matpath, 'publications', 'fastTracking', 'data');
modelPath       = fullfile(matpath, 'AFM_SS');
controlDataPath = fullfile(expDataRoot, controlParamName);
dataOut_path    = fullfile(expDataRoot, 'exp01outputBOTH.csv');
refTrajPath   = fullfile(expDataRoot, refTrajName);
expName       = ['16-Apr-2016_exp001111'];
expName       = ['22-Jun-2016_exp01'];
% expName       = ['11-May-2016_exp01'];

modFitName    = [expName, '.mat'];
modFitPath    = fullfile(dataroot, modFitName);
load(modFitPath, 'modelFit')
modelFit.frf_data = modelFit.frf; %Make compatible with old scheme, and current code.
FitNum    = 'FitNum001';
FitNum    = 'HighFidelity7';
%%
clear vi; clear e;
TOL = .01;
w_s       = modelFit.frf_data.w_s;

SYS   = modelFit.(FitNum);
Ts        = SYS.Ts;
%--------------------------------------------------------------------------
% Build models 
Nd              = SYS.InputDelay;
% decH   = decimateTF(G);
% pairdH = pairDecimated(decH);
% SYS    = ss(pairdH);


% We will create three systems here:
% 1) sys_designK: the S.S. system with delay absorbed as input delay
%    Use this to create the augmented system, (2).
% 2) sys_designK_aug: same as (1) but we augment with integral state.
%    Use this to design K.
% 3) sys_obs: This is the observer system for simulation. It only includes
%    Ns states, with NO delay, because we are implementing a reduced order
%    observer.

% 1). Create system with delay but without integral action
PLANT_NOM = absorbDelay(SYS);

z = [0.9200 + 0.1105i, 0.9200 - 0.1105i];
p = [0.9820 + 0.1593i, 0.9820 - 0.1593i,... % [1 -1.964 0.9897]
     0.9485 + 0.0570i, 0.9485 - 0.0570i,... % [1 -1.897 0.9029]
     0.9080 + 0.2639i, 0.9080 - 0.2639i,... %[1 -1.816 0.8941]
     zeros(1,10)];
k = 2.9238e-4 
PLANT = zpk(z, p, k, Ts)

% PLANT = PLANT_NOM;

num = 0.74736*conv(conv([1 1], [1 -1.816 0.8284]), [1 -1.959 0.9864]);
den = conv(conv(conv([1 0.7072], [1 -0.1]), [1 -1]), [1 -1.882 0.9019]);
den = conv(conv(conv([1 0.7072], [1 -0.7899]), [1 -1]), [1 -1.882 0.9019]);
DD = tf(num, den, Ts);

Ts = 1/25e3;
Dz = zpk(tf(num, den, Ts))

F_CL = zpk([],[],0, Ts);

N    = 4000;
trun = Ts*N;
t = [0:1:N]'*Ts;

F1 = figure(50);

ref_f = 1;
ref_0 = 0;
yref = 0*t+ref_f;

uKx = yref;
[ref_traj, uKx, y_uKx] = pack_uKx_y(uKx, yref, t);

% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -----------------------Simulate and Plot---------------------------------
% 3) set sensor noise level. 


umax = 5;

sim(fullfile(modelPath, 'AFM_TF_fdbk.slx'))

y_TF = yplant; 
u_TF = u_full;

% TFParams = struct('num', num, 'den', den, 'TOL', TOL, 'ref_f', ref_f);
TFParams = struct('num', num, 'den', den);
TFSimOpts = stepExpOpts('TOL', TOL, 'y_ref', ref_f, 'pstyle', '--k',...
                     'name', 'H_{inf} Simulation', 'controller', TFParams);

% exp_TF_sim = stepExp(y_TF, u_TF, 'b', 'H_{inf}, simulation',...
%                      'TF controller', TFParams);
exp_TF_sim = stepExp(y_TF, u_TF, TFSimOpts);

H1 = plot(exp_TF_sim, F1);
%%
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% Setup and run experiment with transfer function
tffforder = 1;
num_ff = 1;
den_ff = 1;
ff_flag = 0;
clc
% Iters = N;
Iters = 1600;
tforder = length(den)-1;
SettleTicks = 20;
dlmwrite(refTrajPath, ref_traj.signals.values, 'delimiter', ',', 'precision', 12);

% -----------------------RUN THE EXPERIMENT--------------------------------
vipath = 'C:\Users\arnold\Documents\labview\ACC2017_archive\play_AFM_TF.vi';
% vipath = 'C:\Users\arnold\Documents\labview\AFM_ss_singlematrix\play_AFM_TF.vi';
[e, vi] = setupVI(vipath, 'SettleTicks', SettleTicks, 'TsTicks', 1600,...
               'Iters', Iters, 'umax', umax, 'num', num, 'den', den, 'TF Order', tforder,...
               'TF ff Order', tffforder, 'num_ff', num_ff, 'den_ff', den_ff, 'ff_on', ff_flag,...
               'reference traj path',refTrajPath, 'outputData BOTH', dataOut_path );
vi.Run
% -------------------------------------------------------------------------


AFMdata = csvread(dataOut_path);
[y_exp, u_exp, xTF] = unpackExpTFData(AFMdata, Ts);

TFExpOpts = stepExpOpts('pstyle', 'r', 'name', 'AFM Stage');

% exp_TF_afm = stepExp(y_exp, u_exp, '--k', 'H_{inf}, AFM Stage', 'linear', TFParams);
exp_TF_afm = stepExp(y_exp, u_exp, TFExpOpts);

H2 = plot(exp_TF_afm, F1);


%%
systems.PLANT = PLANT;
systems.Dz    = Dz;
save(fullfile(expDataRoot, '09_19_2016_Hinf_01.mat'), 'exp_TF_sim', 'exp_TF_afm', 'systems')

