% This class is to store feedback gains and methods.


classdef fdbk
    %FDBK Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        K;
        Q;
        R;
        gams;
        alps;
        pint;
        rhos;
        method;
        Pdes;
        rad;
        doDelay;
        obs;
        notes;
    end
    
    methods
        function obj = fdbk(SYS, varargin)
            obj.obs = 0;
            
            if length(varargin) <= 6 && ~(length(varargin)>6)
                fprintf('assuming LQR method \n')
                meth = 'LQR';
                opts = struct('Q', 1, 'R', 1, 'obs', 0);
            elseif length(varargin) <= 12 & length(varargin)>6
               fprintf('assuming acker method \n')
               meth = 'acker';
               opts = struct('gams', 1, 'alps', 1,...
                'pint', 1, 'rhos', 1, 'obs', 0, 'doDelay', 0, 'rad', 0);
            else
                length(varargin)
                error('Invalid number of inputs')
            end
            
            
            obj.method = meth;
%             keyboard
            optNames = fieldnames(opts);
            
            % Set defaults first.
            for i = 1:length(optNames)
                obj.(optNames{i}) = opts.(optNames{i});
            end
            
            % Now set fields to varargin inputs.
            for pair = reshape(varargin, 2, [])
                inpName = (pair{1});
               if any(strcmp(inpName, optNames))
                obj.(inpName) = pair{2};
               else
                    error('%s is not note a recognized parameter name', inpName)
               end
            end
            
            if obj.obs == 1
                SYS = getDual(SYS);
            end
            if strcmp(obj.method, 'LQR')
                obj.K = dlqr(SYS.a, SYS.b, obj.Q, obj.R);
            elseif strcmp(obj.method, 'acker')
               obj.Pdes  =  getCharDes(SYS, obj.gams, obj.pint, obj.alps, obj.rhos, obj.rad);
               if obj.doDelay
                   obj.K     = place(SYS.a, SYS.b, obj.Pdes);
               else
                   obj.K     =  acker(SYS.a, SYS.b, obj.Pdes);
               end
            end
            
            if obj.obs == 1
                obj.K = obj.K';
            end
            
            
        end % end contsructor.
    end
    
end


% % given radius r and Nd, generate evenly spaced circle of delay poles. 
% function delPls = getDelayPls(Nd, r)
%     Nd_even = floor(Nd/2)+1
%     div = pi/Nd_even
% 
%     thets = div:div:pi-div
%     rs    = 0*thets + r;
%     p1 = rs.*cos(thets) + rs.*sin(thets)*j;
% 
%     P = [p1, conj(p1)]
% 
%     if floor(Nd/2) ~= Nd/2
%         P = [P, 0]
%     end
% end

% Create Dual system.
function sys = getDual(sys)
b = sys.b;
sys.a = sys.a';
sys.b = sys.c';
sys.c = b';

end


