% [Nbar, Nx, Nu] = getNbar(sys, K)
% For square systems systems only, ie, no=ni.
function [Nbar, Nx, Nu] = getNbar(sys, K)
    szC = size(sys.c);
    szB = size(sys.b);
    no = szC(1);
    ni = szB(2);
    if ni~=no
        error('System should be square. This script requires no=ni')
    end
    if nargin <3
        Nd_in = 0;
    end
    if nargin < 4
         Nd_out = 0;
    end

    Ns = length(sys.b);


    if sys.Ts ~=0

        Q    = [sys.a-eye(Ns+Nd_in+Nd_out), sys.b;
                sys.c,                            0*sys.d];
        % NxNu = inv(Q)*[0; 1].
        NxNu = Q\[zeros(Ns+Nd_in+Nd_out, no); eye(no)];
        
        % Pull out Nx and Nu.
        Nx   = NxNu(1:Ns+Nd_in+Nd_out,:);
        Nu   = NxNu(end-no+1:end,end-no+1:end);
        % Compute Nbar.
        if isempty(K)
            Nbar = [];
        else
            Nbar = Nu + K*Nx;              
        end

    else
        Q = [sys.a, sys.b;
            sys.c,  sys.d];
        NxNu = Q\[zeros(Ns+Nd_in+Nd_out, no); eye(no)];
        
        % Pull out Nx and Nu.
        Nx   = NxNu(1:Ns+Nd_in+Nd_out,:);
        Nu   = NxNu(end-no+1:end,end-no+1:end);
        % Compute Nbar.
        Nbar = Nu + K*Nx;
        if isempty(K)
            Nbar = [];
        else
            Nbar = Nu + K*Nx;              
        end
        
        

    end


end