% [Q, R, KKK] = inverseLQR(sys, K)
%
% Tries to solve the inverse LQR problem for a discrete time system. 
% Note, the problem is not always feasible.
%
% Inputs: 
%   sys: discrete time state space system. 
%   K: control gain
%
% Outputs:
%   (Q,R) s.t. lqr(sys, Q,R) = K
%   KKK: KKK = lqr(sys, Q, R). If K ~= KKK, then the problem is infeasible.

function [Q, R, KKK] = inverseLQR(sys, K)


if sys.Ts ==0
    error('System should be discrete time, but Ts=0')
end

Az = sys.a;
Bz = sys.b;

Ns = length(Bz);


alp = 0;
PP = sdpvar(Ns,Ns);
P1 = sdpvar(Ns,Ns);
QQ = sdpvar(Ns,Ns);
% RR = sdpvar(1,1);
R = 1;
F = [(PP >= 0),  (P1 >= 0)];

F = F + [(QQ >= alp)];%, (RR >= alp)];
F = F + [((Az-Bz*K)'*PP*(Az-Bz*K) -PP + K'*R*K + QQ == 0)];
F = F + [(Bz'*PP*Az - (R+Bz'*PP*Bz)*K == 0)];
F = F + [(Az'*P1*Az - P1 <= QQ)];

% keyboard
optimize(F);

Q = double(QQ);

KKK = dlqr(Az, Bz, Q, R);
end