% [y_ref, u_kx, y_uKx] = pack_uKx_y(uKx, y, t)
% 
% Builds timeseries versions of y, and uKx for use in simulink.
%
% Also, packs y and uKx into a single vector for use in LabView. This
% vector has the form
%  y_uKx = [uKx(1); y(1); uKx(2); y(2) ....]
% 
%   Inputs:
%       uKx: vector of feedforward inputs, e.g. , u_des + K*xdes
%       y:   vector of y_desired, e.g. for integral tracking.
%       t:   vector of time.
%   Outputs:
%       y_ref: timeseries. y_ref.signals.values = y, y_ref.time = t;
%       u_Kx:  timeseries. u_Kx.signals.values = uKx, u_Kx.time = t;
%       y_uKx: a vector s.t. [uKx(1); y(1); uKx(2); y(2) ....], length is
%       2*length(t).


function [y_ref, u_Kx, y_uKx] = pack_uKx_y(uKx, y, t)


y_ref.signals.values = y;
y_ref.time           = t;

u_Kx.signals.values = uKx;
u_Kx.time           = t;

y_uKx                = [];
for i = 1:length(t)
   y_uKx = [y_uKx;  u_Kx.signals.values(i); y_ref.signals.values(i);];
end

end

