% 
%  Inputs:
%       H, a decimateTF object.
%  Properties:
%       realG;
%       cmplxG;
%       delG;
%       K;
%
%  Methods:
%       Constructor
%       ss(obj)
%       zpk(obj)
classdef pairDecimated
   properties
      realG;
      cmplxG;
      delG;
      K;
       
   end
    
   methods
       function obj = pairDecimated(H) 
          for i = 1:length(H.rz.wn)
              [~, k_i] = min(abs(H.rp.wn - H.rz.wn(i)) );
              try
                  realG(i) = H.rz.G(i)*H.rp.G(k_i);
              catch
                  realG(i) = H.rz.G(i);
              end
              % now, delete them from the running for next iter...
              H.rp.G(k_i) = [];
              H.rp.wn(k_i) = [];
          end
         for i = 1:length(H.rp.wn)
             realG = [realG, H.rp.G(i)];
         end
           obj.realG = realG;
           
           %-----------------------
           
           for i = 1:length(H.cz.wn)
              [~, k_i] = min(abs(H.cp.wn - H.cz.wn(i)) );
              cmplxG(i) = H.cz.G(i)*H.cp.G(k_i);
              % now, delete them from the running for next iter...
              H.cp.G(k_i) = [];
              H.cp.wn(k_i) = [];
           end
          
         for i = 1:length(H.cp.wn)
             cmplxG = [cmplxG, H.cp.G(i)];
         end

           obj.cmplxG = cmplxG;
           obj.delG    = H.del;
           obj.K      = H.K;
       end % end Constructor.
       
       function G = zpk(obj)
          G = obj.K*obj.delG;
          for i = 1:length(obj.realG);
              G = G*obj.realG(i);
          end
          for i = 1:length(obj.cmplxG)
              G = G*obj.cmplxG(i);
          end
          
       end % end zpk
       
       function SYS = ss(obj)
          SYS = ss(obj.K);
          
        
          obj = pairLonely(obj);
          for i = length(obj.cmplxG):-1:1
              SYS = SYS*ss(obj.cmplxG(i));
              if ~isempty(SYS.e)
                  fprintf('systems seems non-causal: investigate')
                  keyboard
              end
          end
          for i = 1:length(obj.realG)
              SYS = SYS*ss(obj.realG(i));
              if ~isempty(SYS.e)
                  keyboard
              end
          end
           SYS.InputDelay = order(obj.delG);
       end
           
       
       
       
       
   end % end methods.
    
    
end


function lonely_real_s = findLonelyReals(realG)
    lonely_real_s = [];
    for i = 1:length(realG)
       if length(pole(realG(i))) < length(zero(realG(i)))
          lonely_real_s = [lonely_real_s; i]; 
       end
    end

end


function cmplx_mates = findCmplxMates(cmplxG)
    cmplx_mates = [];
    for i = 1:length(cmplxG)
       if length(zero(cmplxG(i))) < length(pole(cmplxG(i))) 
          cmplx_mates = [cmplx_mates; i]; 
       end
    end
end


function pp = pairLonely(pp_old)

lonely_real_s = findLonelyReals(pp_old.realG);
cmplx_mates = findCmplxMates(pp_old.cmplxG);

  if length(lonely_real_s) ==0
%     return;
  elseif length(cmplx_mates)*2 < length(lonely_real_s)
                      % *2 because each complex transfer fcn can have two
                      % real zeros added to it before it becomes acausal.
      error('the system seems non-causal')
      
  else
      

%     if length(lonely_real_s) > length(cmplx_mates)
%         cmplx_iter = 1;
%         
%         for i = 1:length(lonely_real_s)
%         
%             pp_old.cmplxG(cmplx_mates(cmplx_iter)) =  pp_old.cmplxG(cmplx_mates(cmplx_iter))*pp_old.realG(lonely_real_s(i));
%            pp_old.realG(lonely_real_s(i)) = [];
%            if tfFull(pp_old.cmplxG(cmplx_mates(cmplx_iter)))
%                cmplx_iter = cmplx_iter + 1;
%            end
%         end
%     else % Normal case
        for i = 1:length(lonely_real_s)
            pp_old.cmplxG(cmplx_mates(i)) =  pp_old.cmplxG(cmplx_mates(i))*pp_old.realG(lonely_real_s(i));
           pp_old.realG(lonely_real_s(i)) = [];
        end % for
%     end % if

      
%     try
%     if length(lonely_real_s) > length(cmplx_mates)
%         cmplx_iter = 1;
%         
%         for i = 1:length(lonely_real_s)
%         
%             pp_old.cmplxG(cmplx_mates(cmplx_iter)) =  pp_old.cmplxG(cmplx_mates(cmplx_iter))*pp_old.realG(lonely_real_s(i));
%            pp_old.realG(lonely_real_s(i)) = [];
%            if tfFull(pp_old.cmplxG(cmplx_mates(cmplx_iter)))
%                cmplx_iter = cmplx_iter + 1;
%            end
%         end
%     else % Normal case
%         for i = 1:length(lonely_real_s)
%             pp_old.cmplxG(cmplx_mates(i)) =  pp_old.cmplxG(cmplx_mates(i))*pp_old.realG(lonely_real_s(i));
%            pp_old.realG(lonely_real_s(i)) = [];
%         end % for
%     end % if
% 
%     catch
%       keyboard
%     end
      
  end
  
  pp = pp_old;
  
  
  

end


% Determine if the transfer function is "Full", ie, cannot add anymore
% zeros before it becomes acausal.
function tfFullFlag = tfFull(G)
    [num, den] = tfdata(G, 'v')
    if length(pole(G)) == length(zero(G))
        tfFullFlag = 1;
    else 
        tfFullFlag = 0;
    end
end

