% pertSYS = perturbsys(SYS, gam_p, gam_z, alp_p, alp_z, dc_true)
% Given a second order system (discrete or cont), this script will change the natural
% frequency by the amount gam_p and gam_z, and the damping by the amount alps.
% 
% E.G, for 
%  G(s) = s +  z
%         -------
%         s + p
% 
%  We have 
%         p <-- gam_p
%         z <-- gam_z
%
% If G(s) has only a zero at infinity, then gam_z is not needed.
% 
%  Finally, dc_true is a flag. If 1, then rescale dcgain.



function pertSYS = perturbsys2(SYS, gam_p, gam_z, dc_true)
[num, den] = tfdata(SYS, 'v');
    if length(den) ~= 2
        error ('This function is only defined for a first order system')
    end
    Ts = SYS.Ts;        
    dcg = dcgain(SYS);
    [z, p, k] = zpkdata(SYS, 'v');
    
    if SYS.Ts ~= 0
            % If discrete, map to s-plane.
        z = log(z)/Ts;
        p = log(p)/Ts;
    end
    
    
    % Get Naturual frequency and damping of poles.
    wp = abs(p(1));
    zp = -real(p(1))/wp;
    
    % Perturb Poles.
    p = (-wp*gam_p);
    
    
    % Same for zeros. But accept single or double complex poles. 
    if ~isempty(z)
            z = gam_z*z;
    end

if SYS.Ts ~= 0
    z = exp(z*Ts);
    p = exp(p*Ts);
end
    
% keyboard
if dc_true
    sys2 = zpk(z, p, k, Ts);
    
    dc2 = dcgain(sys2);
    pertSYS = sys2*(dcg/dc2);
else
    pertSYS = zpk(z, p, k, Ts);    
end


end