%  [T_S, k_s] = settle_time(T0, yy, VAL, TOL, plots)
%
%  Calculate the time for a signal to settle to within a Tolerance of a
%  desired value.
%
%
%  T0   = time vector.
%  yy   = signal
%  TOL  = desired Tolereance of desired value: NOT A PERCENT!!
%  VAL  = desired value.
%
%
%  Returns:
%  TS = settle time
%  k   = settle time index
%
% Optional
% 
% plots = 0 --> no plots, 1 --> plots
% fignum = figure number for plots
function [TS, k_s] = settle_time(T0, yy, VAL, TOL, fignum, subfignum)




yy_norm = yy - VAL;

yy_b = flipud(yy_norm);

if (max(abs(yy_norm(end-4:end))) > TOL) % Trying to prevent data that just happens to 
                                        % fall with settle boundary at very
                                        % end of data vector
    
    TS = NaN;
    k_s = NaN;
else
    k_s = find(abs(yy_norm)>TOL, 1, 'last')+1;
%     k_b = find(abs(yy_b) > TOL,  1, 'first');
%     k_s = length(yy) - k_b;
    TS  = T0(k_s);
end



if (exist('fignum') && isfinite(TS))
    figure(fignum)
    if exist('subfignum', 'var')
        subplot(2,1,1)
    end
    plot(T0, yy)
    hold on
    plot([T0(1), T0(end)], [VAL+TOL, VAL+TOL], '--')
    plot([T0(1), T0(end)], [VAL-TOL, VAL-TOL], '--')
    plot(T0(k_s), yy(k_s), 'xk')
end



end