% [y_exp, u_exp, xhat_exp xdelay_exp] = unpackExpTFData(data,  Ts)
% 
% Unpacks the experimental data that LabView spits out into pieces that
% make sense.
%
% Inputs: 
%   data: from labview. We expect the following format:
%       dataBoth = [y, u, X1.....Xns,  ]
%   
%
% Outputs:
%   y_exp, u_exp. Actual measured input/output to AFM.
%   xhat_exp: the estimated states from the observer.
%   xdelay_exp: Input delay states, from shift register. 
%
%   These are all structs, e.g. y_exp.Data, y_exp.Time.


function [y_exp, u_exp, xhat_exp xdelay_exp] = unpackExpTFData(data, Ts)
    y_exp    = timeseries();
    u_exp    = timeseries();
    xhat_exp = timeseries();
    
    % Make Data Accesible
    % dataBoth = [y,  X1.....Xns,  u,  Xd]
    y_exp.Data = data(:,1);
    ty = (0:Ts:(length(y_exp.Data)-1)*Ts)';
    y_exp.Time = ty;
    
    u_exp.Data = data(:, 2);
    tu = (0:Ts:(length(u_exp.Data) - 1)*Ts)';
    u_exp.Time = tu;
    
    
    % Pull out observer state data.    
    xhat_exp.Data   = data(:, 3:end);
    xhat_exp.Time   = (0:Ts:(length(xhat_exp.Data(:,1))-1)*Ts)';

    
end