% [ref_f, ref_0, x0, xf, xss] = uss2yss(sys, uss_f, uss_0)
%
% Given a state-space sytem, sys, maps final and initial steady state
% control inputs, uss_f, uss_0, into steady state values of y (ie,
% reference). Also, outputs steady state state vector, x0, xf.
%
% If uss_0 is ommitted, function assumes uss_0 = 0.
%
% See also yss2uss.
%
% Arnold Braker
% 06-14-2016


function [ref_f, ref_0, x0, xf, xss] = uss2yss(sys, uss_f, uss_0)

if nargin <3
    uss_0=0;
end

if ~isa(sys, 'ss')
    error('sys must be a state space system')
end


Ns = length(sys.b);
if sys.Ts == 0
    xss = -inv(sys.a)*sys.b
else
    xss = (eye(Ns)-sys.a)\sys.b;
end

ref_f     = (sys.c*xss)*uss_f;
ref_0     = (sys.c*xss)*uss_0;

x0      = xss*uss_0;
xf      = xss*uss_f;



end