% [uss_0, uss_f, x0, xf, xss] = yss2uss(sys, ref_f, ref_0)
% 
% Given a state space system, maps an initial and final (steady state)
% reference into the appropriate steady state control inputs. ALso returned
% are the respective steady state values of x0 and xf.
%
% If ref_0 is omitted, the function assumes ref_0 =0.
% 
% See also uss2yss.
%
% Arnold Braker
% 06-14-2016


function [uss_0, uss_f, x0, xf, xss] = yss2uss(sys, ref_f, ref_0)

if nargin <3
    ref_0=0;
end



if ~isa(sys, 'ss')
    error('sys must be a state space system')
end


Ns = length(sys.b);
if sys.Ts == 0
    xss = -inv(sys.a)*sys.b
else
    xss = (eye(Ns)-sys.a)\sys.b;
end

uss_f     = ref_f/(sys.c*xss);
uss_0     = ref_0/(sys.c*xss);

x0      = xss*uss_0;
xf      = xss*uss_f;

end