clear
clc
close all

fig1_name = 'clqr01_step_trk2.eps';
fig2_name = 'clqr01_pzplot.eps';

dataroot      = 'data';
figpath       = getFigPath();

load(fullfile(dataroot, '09-12-2016_loopCLQR_trk2_extend_01.mat'));

fig1_path = fullfile(figpath, fig1_name);
fig2_path = fullfile(figpath, fig2_name);

Ts = 1/25e3;
volts2um = 50/10;
sim_exp = the_exp.sims;


afm_exp = the_exp.afm;


% Change some of the parameters:
sim_exp.pstyle = '--k';
sim_exp.name   = 'Simulation';
sim_exp.yscaling = volts2um;
sim_exp.yunits = '[$\mu$ m]'; 
sim_exp.datanames = {'y(k)'};

afm_exp.pstyle = 'r';
afm_exp.name   = 'Piezo Stage';
afm_exp.yscaling = volts2um;
afm_exp.yunits = '[$\mu$ m]'; 
afm_exp.datanames = {'y(k)'};

F1 = figure(1);
F1.Position = [-1919 374 1004 420];


H2 = plot(afm_exp, F1, 'textOn', 0);
H1 = plot(sim_exp, F1, 'textOn', 0);


F1.Children(1).YLim = [-3 3];
F1.Children(1).YTick = [-3 -2 -1 0 1 2 3];
F1.Children(1).XLim  = [0, afm_exp.u.Time(end)];
F1.Children(1).YLabel.String = 'u(k) [v]';

F1.Children(2).XLim  = [0, afm_exp.u.Time(end)];
F1.Children(2).YLabel.String = 'y(k) [$\mu$m]';

leg1 = legend([H1(2), H2(2)]);
leg1.FontSize = 12;
% leg1.Position = [0.6583 0.7421 0.2268 0.0857];
leg1.Interpreter = 'latex';


% calc settle times
ref1 = afm_exp.params.ref_f;
ref2 = afm_exp.params.ref_f_2;
TOL1 = abs(0.01*ref1);
TOL2 = abs(0.01*ref2);
ts1_sim = settle_time(sim_exp.y.Time(1:800), sim_exp.y.Data(1:800), ref1, TOL1)
ts1_afm = settle_time(afm_exp.y.Time(1:800), afm_exp.y.Data(1:800), ref1, TOL1)


ts2_sim = settle_time(sim_exp.y.Time(802:end), sim_exp.y.Data(802:end), ref2, TOL2)-Ts*802
ts2_afm = settle_time(afm_exp.y.Time(802:end), afm_exp.y.Data(802:end), ref2, TOL2)-Ts*802
%%
try
    load('metadata.mat', 'data');
end
data.clqr_ts_data.ts1_sim = ts1_sim;
data.clqr_ts_data.ts2_sim = ts2_sim;
data.clqr_ts_data.ts1_afm = ts1_afm;
data.clqr_ts_data.ts2_afm = ts2_afm;

save('metadata.mat', 'data')

%%
subplot(2,1,1)
ylim([-8, 12]);
ylm = ylim;

plot([400 400]*Ts, [ylm(2), ref1*volts2um], ':k');
plot([808 808]*Ts, [ylm(2), ref1*volts2um], ':k');
plot([1200 1200]*Ts, [ylm(2), ref2*volts2um], ':k');
plot([1600 1600]*Ts, [ylm(2), ref2*volts2um], ':k');

ya1 = .88;
ys1 = .94;
ax1 = [.129, .323];
ay = [ya1, ya1];
a1 = annotation('doublearrow', ax1, ay, 'Units', 'Normalized');

s1 = text(.05, ys1, 'FIFO Active (a)', 'interpreter', 'latex', 'units', 'normalized', 'FontSize', 12);

ax2 = [.323, .521];
ay2 = [ya1 ya1];
a2 = annotation('doublearrow', ax2, ay2, 'Units', 'Normalized');
s2 = text(.3, ys1, 'FIFO Empty (b)',...
          'interpreter', 'latex', 'units', 'normalized', 'FontSize', 12);

ax3 = [.521, .711];
ay3 = [ya1 ya1];
a3 = annotation('doublearrow', ax3, ay3, 'Units', 'Normalized');
s3 = text(.55, ys1, 'FIFO Active (c)',...
          'interpreter', 'latex', 'units', 'normalized', 'FontSize', 12);

ax4 = [.711, .905];
ay4 = [ya1 ya1];
a4 = annotation('doublearrow', ax4, ay4, 'Units', 'Normalized');
s4 = text(.78, ys1, 'FIFO Empty (d)',...
          'interpreter', 'latex', 'units', 'normalized', 'FontSize', 12);

%%
K_lqr_prime = afm_exp.params.K_fdbkLQR.K;

[~,~, K_lqr] = inverseLQR(the_exp.systems.sys_obs, K_lqr_prime);



sys = the_exp.systems.sys_obs;
sys_cl = ss(sys.a-sys.b*K_lqr, sys.b, sys.c, 0, sys.Ts);

pls_ol = pole(sys);
zrs_ol = zero(sys);
pls_cl = pole(sys_cl);

F2 = figure(2);clf
hold on;
ylim([-.3, .3])
xlim([.7,1.])

h1 = plot(real(pls_ol), imag(pls_ol), 'xb', 'MarkerSize', 10);
h2 = plot(real(zrs_ol), imag(zrs_ol), 'ob', 'MarkerSize', 10);
h3 = plot(real(pls_cl), imag(pls_cl), 'xr', 'MarkerSize', 10);
zgrid


h1.DisplayName = ': $G_1$ Pole $\:\:$';
h2.DisplayName = ': $G_1$ Zero $\:\:$';
h3.DisplayName = ': $(A-BK_{lqr}$) pole $\:\:$';
hleg= legend([h1, h2, h3]);
hleg.Interpreter = 'latex';
hleg.FontSize    = 12;
ylabel('imaginary axis', 'interpreter', 'latex', 'FontSize', 14)
xlabel('real axis', 'interpreter', 'latex', 'FontSize', 14)
hleg.Orientation = 'horizontal';
hleg.Position    = [0.1542 0.9281 0.7178 0.0604];
%%
hgexport(F1, fig1_path, ...
    hgexport('factorystyle'), 'Format', 'eps')
%%
hgexport(F2, fig2_path, ...
    hgexport('factorystyle'), 'Format', 'eps')
