clear
clc
close all

fig1_name = 'linearslew_trk2.eps';
fig2_name = 'linear_pzplot.eps';


dataroot = 'data'; 
figpath  = getFigPath();
load(fullfile(dataroot, '09_12_2016_linfdbk_slewproblem_trk2.mat'));
afm_exp_15 = afm_exp;

load(fullfile(dataroot, '09_11_2016_linfdbk_slewproblem_trk2.mat'));

fig1_path = fullfile(figpath, fig1_name);
fig2_path = fullfile(figpath, fig2_name);

Ts = 1/25e3;
volts2um = 50/10;

% Change some of the parameters, and fix things to account for changes in 
%  stepExp methods and properties:
sim_exp.pstyle = '--k';
sim_exp.name   = 'Simulation';
sim_exp.yscaling = volts2um;
sim_exp.yunits = '[$\mu$ m]'; 
sim_exp.datanames = {'y(k)'};


afm_exp.pstyle = 'r';
afm_exp.yscaling = volts2um;
afm_exp.yunits = '[$\mu$ m]'; 
afm_exp.datanames = {'y(k)'};

afm_exp.name   = 'Piezo Stage';
afm_exp_15.pstyle = ':k';
afm_exp_15.name   = 'Piezo Stage: $y_{ref} = 8.0~\mu$m'
afm_exp_15.yscaling = volts2um;
afm_exp_15.yunits = '[$\mu$ m]'; 
afm_exp_15.datanames = {'y(k)'};

% ==============================
% Do plotting:
F1 = figure(1);
F1.Position = [-1919 374 1004 420];


H2 = plot(afm_exp, F1, 'textOn', 0);
H1 = plot(sim_exp, F1,  'textOn', 0);
H3 = plot(afm_exp_15, F1,  'textOn', 0);



F1.Children(1).YLim = [-5 6];
F1.Children(1).XLim  = [0, afm_exp.u.Time(end)];
F1.Children(1).YLabel.String = 'u(k) [v]';
F1.Children(2).XLim  = [0, afm_exp.u.Time(end)];
F1.Children(2).YLabel.String = 'y(k) [$\mu$m]';

leg1 = legend([H1(2), H2(2), H3(2)]);
leg1.FontSize = 12;
leg1.Interpreter = 'latex';
% Change ylabels to match paper (needed because of changed stepExp plotting
% routine)

% calc settle times
ref1 = afm_exp.params.ref_f;
ref2 = -ref1;
% ref2 = afm_exp.params.ref_f_2;
TOL1 = abs(0.01*ref1);
TOL2 = abs(0.01*ref2);
ts1_sim = settle_time(sim_exp.y.Time(1:800), sim_exp.y.Data(1:800), ref1, TOL1)
ts1_afm = settle_time(afm_exp.y.Time(1:800), afm_exp.y.Data(1:800), ref1, TOL1)


ts2_sim = settle_time(sim_exp.y.Time(802:end), sim_exp.y.Data(802:end), ref2, TOL2)-Ts*802
ts2_afm = settle_time(afm_exp.y.Time(802:end), afm_exp.y.Data(802:end), ref2, TOL2)-Ts*802

%%
try
    load('metadata.mat', 'data');
end
data.lin_ts_data.ts1_sim = ts1_sim;
data.lin_ts_data.ts2_sim = ts2_sim;
data.lin_ts_data.ts1_afm = ts1_afm;
data.lin_ts_data.ts2_afm = ts2_afm;

save('metadata.mat', 'data')

%%
K   = systems.Kfdbk.K;
sys = systems.sys_designK_aug;
sys_cl = ss(sys.a-sys.b*K, sys.b, sys.c, 0, sys.Ts);

pls_ol = pole(sys);
zrs_ol = zero(sys);
pls_cl = pole(sys_cl);

F2 = figure(2);clf
hold on;

h1 = plot(real(pls_ol), imag(pls_ol), 'xb', 'MarkerSize', 10);
h2 = plot(real(zrs_ol), imag(zrs_ol), 'ob', 'MarkerSize', 10);
h3 = plot(real(pls_cl), imag(pls_cl), 'xr', 'MarkerSize', 10);
zgrid

% ylim([-.3, .3])
xlim([-.5, 1.3])

h1.DisplayName = ': $G_3$ Pole $\:\:$';
h2.DisplayName = ': $G_3$ Zero $\:\:$';
h3.DisplayName = ': $(\Phi_{aug}-\Gamma_{aug}K_{aug})$ pole ';
hleg= legend([h1, h2, h3]);
hleg.Interpreter = 'latex';
hleg.FontSize    = 12;
ylabel('imaginary axis', 'interpreter', 'latex', 'FontSize', 14)
xlabel('real axis', 'interpreter', 'latex', 'FontSize', 14)
hleg.Orientation = 'horizontal';
hleg.Position    = [0.1235    0.9233    0.7934    0.0604];
%%
hgexport(F1, fig1_path, ...
    hgexport('factorystyle'), 'Format', 'eps')

hgexport(F2, fig2_path, ...
    hgexport('factorystyle'), 'Format', 'eps')
